<?php
/*  Copyright (c) 2015 - 2018 ArcLindon LLC, All Rights Reserved
 *  $Id: sseo_lib_utilities.php 397 2019-03-01 23:21:08Z bill $
 *
 *  Changes:   
 *  2019/03/01  bherndon    v12.0.3.6a
 *  2019/03/01  bherndon    Added include for sseo_options.php and function
 *                          sseo_getVersion()
 *  2018/02/10  bherndon    v11.1.1.6 
 *  2018/02/10  bherndon    Keywords now handle any custom event in addition
 *                          to names and nicknames
 *  2018/02/06  bherndon    Now encoding HTML special characters for headers
 *  2017/09/02  bherndon    v11.1.0.5
 *  2017/08/17  bherndon    Corrected rights calculations in fetchHusb() and
 *                          fetchWife() 
 *  2017/02/04  bherndon    v11.1.0.4
 *  2017/02/04  bherndon    Now supporting variable extensions path
 *  2017/01/27  bhenrdon    Added methods for person names with and without
 *                          IDs
 *  2016/10/09  bherndon    v11.0.0.3
 *  2016/10/08  bherndon    Simplified to a single class with custom localized
 *                          text
 *  2016/09/27  bherndon    Sub-classes for Swedish, Norwegian, Danish, and 
 *                          French
 *  2016/09/21  bherndon    Sub-classes for Italian, German, PortugueseBR
 *  2016/08/23  bherndon    Classes for handling localization of family and 
 *                          individual phrases (English, Spanish)
 *  2015/11/01  bherndon    Now stripping HTML tags from names just in case 
 *                          Living Color mod is installed
 *  2015/11/01  bherndon    Fixed blank name generated from nickname assignment
 *  2015/08/29  bherndon    v10.1.0.1
 *  2015/08/29  bhenrdon    Logic to surpress nicknames
 *  2015/08/22  bherndon    Added rights checking code to getNameListFromRow()
 *  2015/08/08  bherndon    v10.1.0.0
 *  2015/08/08  bherndon    Modified event sweep logic to return all events
 *  2015/08/01  bherndon    Created from sseo_lib_simple_titles.php@154 
 *                          (trunk)
 */
 
//  Include the settings
include($cms['tngpath'] . "$extspath/sseo_settings.php");
include($cms['tngpath'] . "$extspath/sseo_options.php");

/*  Returns the current mod version number as a string.
 *  
 *  Returns:    A string
 */
function sseo_getVersion()
{
  global $sseo_version;
  
  return $sseo_version;
}

/*  Given a valid person ID, retrieves a subset of the TNG person record
 *  including the nickname field.
 *  
 *  $tree:      Family tree to fetch from 
 *  $personID:  Person ID
 *
 *  Returns:    A database record
 */
function sseo_getPersonPlus($tree, $personID) {
	global $people_table;

	$query = "SELECT personID, gedcom, firstname, lnprefix, lastname, prefix, 
	                 suffix, nickname, sex, nameorder, living, private, branch, 
	                 birthdate, birthdatetr, altbirthdatetr, deathdate
		        FROM   $people_table
		        WHERE      personID = \"$personID\" 
		               AND gedcom = \"$tree\"";

	return tng_query($query);
}

function sseo_getNameID($row)
{
  global $sseo_person_ids;
  
  return ($sseo_person_ids == 'yes')
         ? sseo_getName($row) . " (" . $row['personID'] . ")"
         : sseo_getName($row);
}

function sseo_getName($row)
{
  return getName($row);
}

/*  Given a valid source ID, adds all the known information about the source
 *  (as GEDCOM tags) to the input associative array.
 *  
 *  $entityId:  A entity (person/family) identifier 
 *  $result:    An array to add results to
 *
 *  Returns:    (none)
 *
 *  Contract:   If there are any duplicates source fields that already exist
 *              in the input array, they will be overridden buy new values from
 *              the sources table.  Effectively, this means that preference is
 *              given to fields from the sources table.
 */
function sseo_getEventsAsTags($entityID) {
	global $events_table, $sources_table, $repositories_table, 
	       $eventtypes_table, $text, $tree;
 
	$result = array();
	$rtncnt = 0;
	
  // prep a query for 'custom' event fields
  if ($tree)
    $query = "SELECT display, info, tag, description, eventID 
              FROM   ($events_table, $eventtypes_table) 
              WHERE      persfamID = \"$entityID\" 
                     AND $events_table.eventtypeID = $eventtypes_table.eventtypeID 
                     AND gedcom = \"$tree\" 
                     AND keep = \"1\" 
                     AND parenttag = \"\" 
              ORDER BY eventID";
	else        
    $query = "SELECT display, info, tag, description, eventID 
              FROM   ($events_table, $eventtypes_table) 
              WHERE      persfamID = \"$entityID\" 
                     AND $events_table.eventtypeID = $eventtypes_table.eventtypeID 
                     AND keep = \"1\" 
                     AND parenttag = \"\" 
              ORDER BY eventID";
  
  // execute the query for custom events
	$custevents = tng_query($query);
	              
	while ($custevent = tng_fetch_assoc($custevents))	{
	  $result[] = $custevent;
	}

  // release the cursor for the custom fields
  tng_free_result($custevents);
  
  // return the events
  return $result;
}

/*  Given a set of TNG events in the form TAGNAME => array of event fields, 
 *  returns all the 'info' fields that correspond to events having a tag that
 *  is found in the input array $tags.  
 *
 *  $events     Associative array of custom event data
 *  $tags       Array of tags to seek (name 'events')
 *  
 *  Returns:    An array of data items extracted from each of the identified
 *              tags.
 */
function sseo_getEventData($events, $tags)
{
  $result = array();
  $rtncnt = 0;
  
  foreach ($events as $event => $data) {
    if (array_key_exists($data['tag'], $tags)) {
      // the current event corresponds to a tag we're interested in
      $result[] = htmlspecialchars($data['info']);
    }
  }
  
  // return the event's 'info' fields
  return $result;
}

/*  Given a TNG person record, finds and returns an array of all names
 *  associated with that individual.  TNG event types that correspond to names
 *  are limited to those specified in the settings file for the Mod.
 *
 *  $row        Data row representing the individual
 *
 *  Returns:    An simple array of all names by which the individual is known.
 *
 *  Contract:   The data row must have at least the given name, surname, and 
 *              person ID.
 */
function sseo_getNameListFromRow($row) {
  global $sseo_alia_tag, $sseo_nick_tag, $sseo_cust_tag, $sseo_tag_list,
         $tngconfig, $nonames;
  
  $result = array();
  $names  = array();
  $tags   = array();
  
  // create associations from the custom event tag list
  $list = explode(' ', $sseo_tag_list);
  foreach ($list as $elem => $tag) {
    $tags[$tag] = null;
  }
  
  // add rights fields
  $rtree = checktree($row['gedcom']);
	$rights = determineLivingPrivateRights($row, $rtree);
	$row['allow_living'] = $rights['living'];
	$row['allow_private'] = $rights['private'];

  // now fetch the proper name--always given-name surname
  $raw = getNameUniversal($row, 1);
  $raw = strip_tags($raw);
  $result[] = htmlspecialchars($raw);
  
  // now do the nickname and custom events
	if ($rights['both']) {  
	  // get names from custom events
    $entityID = $row['personID']; 
    $events = sseo_getEventsAsTags($entityID);
    $names  = sseo_getEventData($events, $tags);
  
    // add the nickname
    if ($row['nickname']) {
      $names[] = htmlspecialchars($row['nickname']);
    }
	}
  
  // return an array of names
  return array_merge($result, $names);
}

/*  Given TNG family record, finds and returns an array of all names 
 *  associated with that family's father and mother.  TNG event types 
 *  that correspond to names are limited to those specified in the settings 
 *  file for the Mod.
 *
 *  $row        Data row representing the family
 *
 *  Returns:    An simple array of all names by which the father and mother are
 *              known.
 *
 *  Contract:   The data row must have at least the two identifiers of the 
 *              father and mother.
 */
function sseo_getNameListFromFamily($row) {
  global $sseo_alia_tag, $sseo_nick_tag, $sseo_cust_tag, $tree;
  
  $result = array();
    
  // fetch the husband row and names
  $hID  = $row['husband'];
  $hres = sseo_getPersonPlus($tree, $hID);
  if (tng_num_rows($hres)) {
    $hrow   = tng_fetch_assoc($hres);
    $hnames = sseo_getNameListFromRow($hrow);
  }
  
  // fetch the wife row and names
  $wID = $row['wife']; 
  $wres = sseo_getPersonPlus($tree, $wID);
  if (tng_num_rows($wres)) {
    $wrow = tng_fetch_assoc($wres);
    $wnames = sseo_getNameListFromRow($wrow);
  }
  
  // free the result cursors
  tng_free_result($hres);
  tng_free_result($wres);

  // merge the results and return them
  $result = array_merge(
              ($hnames) ? $hnames : array(), 
              ($wnames) ? $wnames : array());  
  return $result;
}
  
/*  Produces an individuals birth and death dates in the form
 *  "1895 - 1974"
 *  
 *  $row:     An associative array representing an individual record
 *  
 *  Returns:  A string
 */
function sseo_getShortDates($row) {
  global $dates;
  
  $result = "";
  
  if($row['deathdate']) {
    $deathdate = $row['deathdate'];
    $truedeath = $row['deathdatetr'];
  } else {
    $deathdate = $row['burialdate'];
    $truedeath = $row['burialdatetr'];
  }
  
  $deathyearonly = $row['death'] && $row['death'] . "-00-00" == $truedeath ? true : false;
  $deathstr = displayDate($deathdate);
  $displaydeath = !$row['death'] || $deathyearonly ? $deathstr : getDatePrefix($deathstr) . $row['death'];
  $displaydeath = $displaydeath != "Y" ? $displaydeath : $dates['Y'];

  if($row['birthdate']) {
    $birthdate = $row['birthdate'];
    $truebirth = $row['birthdatetr'];
  } else {
    $birthdate = $row['altbirthdate'];
    $truebirth = $row['altbirthdatetr'];
  }
  $birthyearonly = $row['birth'] && $row['birth'] . "-00-00" == $truebirth ? true : false;
  $birthstr = displayDate($birthdate);
  $displaybirth = !$row['birth'] || $birthyearonly ? $birthstr : getDatePrefix($birthstr) . $row['birth'];
  if( $displaybirth || $displaydeath )
    $result .= "$displaybirth - $displaydeath";
  
  return $result;
}

/*  Produces an individuals birth and death dates in the form
 *  "b. 10 Jun 1895 d. 23 feb 1974"
 *  
 *  $row:     An associative array representing an individual record
 *  
 *  Returns:  A string
 */
function sseo_getLongDates($row) {
  global $text;
  
  $result = "";
  
  if($row['birthdate'])
    $result .= "{$text['birthabbr']} " . displayDate($row['birthdate']) . " - ";
  else 
    $result .= "? - ";
  if($row['deathdate'])
    $result .= "{$text['deathabbr']} " . displayDate($row['deathdate']);
  else 
    $result .= "?";
  
  return $result;
}

/*  Returns a long-format or short-format individual's birth and
 *  death dates depending on the content of the var $format.  The
 *  global var $sseo_format is used when $format is omitted in 
 *  the call.
 *  
 *  $row:     An associative array representing an individual record
 *  $format:  (optional) A format string ('long' or 'short')
 *  
 *  Returns:  A formatted date string or year
 */
function sseo_getTitleDates($row, $format = null) {
  global $sseo_ind_format;
  
  if ($format)
    if ($format == "short")
      $result = sseo_getShortDates($row);
    else 
      $result = sseo_getLongDates($row);
  else 
    if ($sseo_ind_format == "short")
      $result = sseo_getShortDates($row);
    else 
      $result = sseo_getLongDates($row);
    
  return $result;
}

/*  Given a family record, returns an instance of PhraseClass that can
 *  return family and individual phrases localized for the current language.
 *  
 *  $row:     An associative array representing an TNG family record
 *  
 *  Returns:  A pre-initialized instance of PhraseClass
 */
function sseo_getPhraseInstance($row)
{
  global $language;
  
  return new PhraseClass($row); 
}

/*  Abstract PHP class defining the interface for localized phrases for
 *  families and individuals.
 */
class PhraseClass
{
  // TNG DB person result row
  private $wrow;
  
  // TNG DB person result row
  private $hrow;
  
  // TNG DB family row
  private $source;
    
  function __construct($row) 
  {
    global $tree;
    
    $this->source = $row;
  }

  // Fetch the wife's DB row and assigns to the private var $wrow
  private function fetchWife()
  {
    global $people_table;
    
    $wquery     = "SELECT firstname, lnprefix, lastname, title, prefix, suffix, living, private, branch, nameorder, gedcom FROM $people_table WHERE personID = \"{$this->source['wife']}\" AND gedcom = \"{$this->source['gedcom']}\"";
    $wresult    = tng_query($wquery) or die ($text['cannotexecutequery'] . ": $wquery");
    $this->wrow = tng_fetch_assoc($wresult);

	  $righttree   = checktree($this->source['gedcom']);
    $rightbranch = $righttree ? checkbranch($this->wrow['branch']) : 0;
    $rights = determineLivingPrivateRights($this->wrow, $righttree, $rightbranch);
    $this->wrow['allow_living'] = $rights['living'];
    $this->wrow['allow_private'] = $rights['private'];
		$this->wrow['both'] = $rights['both'];

    tng_free_result($wresult);
  }
  
  // Fetch the husband's DB row and assigns to the private var $hrow
  private function fetchHusb()
  {
    global $people_table;
    
    $hquery = "SELECT firstname, lnprefix, lastname, title, prefix, suffix, living, private, branch, nameorder, gedcom FROM $people_table WHERE personID = \"{$this->source['husband']}\" AND gedcom = \"{$this->source['gedcom']}\"";
    $hresult = tng_query($hquery) or die ($text['cannotexecutequery'] . ": $hquery");
    $this->hrow = tng_fetch_assoc($hresult);
  
	  $righttree   = checktree($this->source['gedcom']);
    $rightbranch = $righttree ? checkbranch($this->hrow['branch']) : 0;
    $rights = determineLivingPrivateRights($this->hrow, $righttree, $rightbranch);
    $this->hrow['allow_living'] = $rights['living'];
    $this->hrow['allow_private'] = $rights['private'];
		$this->hrow['both'] = $rights['both'];
  
    tng_free_result($hresult);
  }
  
  /*  Set's the value of the internal 'wife' row to the input.
   *  
   *  $row:     An associative array representing an person record
   */
  public function setWRow($row)
  {
    $this->wrow = $row;
  }
  
  /*  Set's the value of the internal 'husband' row to the input.
   *  
   *  $row:     An associative array representing an person record
   */
  public function setHRow($row)
  {
    $this->hrow = $row;
  }
  
  /*  Expects an family record and stores it in the instance as the source.  
   *  Has the side-effect of resetting husband and wife person records to 
   *  force a DB refetch for these.
   */
  public function setSource($row)
  {
    $this->source = $row;
    $this->wrow = null;
    $this->hrow = null;
  }
  
  /*  Returns the 'wife' name
   *  
   *  Returns:    A string
   */
  public function getWifeName()
  {
    if (! $this->wrow) {
      $this->fetchWife();
    }

    return getName($this->wrow);
  }
  
  /*  Returns the 'husband' name
   *  
   *  Returns:    A string
   */
  public function getHusbName()
  {
    if (! $this->hrow) {
      $this->fetchHusb();
    }
    
    return getName($this->hrow);
  }
    
  public function formatId($id)
  {    
    return ($id) ? "($id)" : "";
  }
  
  public function getIdStr()
  {
    return ($this->source) 
           ? $this->formatID($this->source['familyID'])
           : "";
  }
  
  /*  Returns a localized family name of the form:
   *    <husband> <conjunct> <wife>
   *
   *  Returns:    A string
   */
  public function getFamilyName()
  {
    global $text;
    
    $result = "";
    $h      = $this->getHusbName();
    $w      = $this->getWifeName();
    
    if (empty($h)) {
      // husband name is empty
      if (! empty($w)) {
        // return just the wife
        $result = $w;
      } else {
        // both are empty - return the ID instead
        $result = $this->getIdStr();
      }
    } else {
      // husband name is not empty
      if (empty($w)) {
        // return just the husband
        $result = $h;
      } else {
        // neither are empty
        $temp   = str_replace('&PARAM1', $h, $text['sseo_familyname']);
        $result = str_replace('&PARAM2', $w, $temp);       
      }
    }
    
    return $result;    
  }
  
  /*  Returns a localized family name of the form:
   *    <introl phrase> <husband> <conjunct> <wife>
   */
  public function getFamilyTitle()
  {
    global $text;
    
    $famname = $this->getFamilyName();
    $result = str_replace('&PARAM1', $famname, $text['sseo_familytitle']);
    
    return $result;
  }
  /*  Returns a localized family name and appends a formatted family ID when
   *  the global variable $sseo_family_ids equals 'yes'.  Otherwise simply 
   *  returns the localized family name.
   *
   *  Family names have the form:  <husband> <conjunct> <wife>
   */
  public function getFamilyNameID()
  {
    global $sseo_family_ids;
    
    $result  = "";
    $famname = $this->getFamilyName();
    
    if ($sseo_family_ids == "yes") {
      // add family IDs
      $result = (empty($famname))
                ? $this->getIdStr()
                : "$famname " . $this->getIdStr();
    } else {
      // drop family IDs
      $result = $famname;
    }
    
    return $result;  
  }
  
  /*  Returns a localized family title and appends a formatted family ID when
   *  the global variable $sseo_family_ids equals 'yes'.  Otherwise simply 
   *  returns the localized family title.
   *
   *  Family titles have the form:
   *    <intro phrase> <husband> <conjunct> <wife> [(<ID>)]
   *  ...as in:
   *    'Familia de Husband y Wife (F306)
   */
  public function getFamilyTitleID()
  {
    global $sseo_family_ids;
    
    $result   = "";
    $famtitle = $this->getFamilyTitle();
    
    if ($sseo_family_ids == "yes") {
      // add family IDs
      $result = "$famtitle " . $this->getIdStr();
    } else {
      // drop family IDs
      $result = $famtitle;
    }
    
    return $result;
  }
}
?>